'use client'

import { FormEvent, useState } from 'react'
import {
  PopoverForm,
  PopoverFormButton,
  PopoverFormCutOutLeftIcon,
  PopoverFormCutOutRightIcon,
  PopoverFormSeparator,
  PopoverFormSuccess,
} from '../../components/popover-form'
import type { Form as ContactFormData, ContactFormBlock as ContactFormProps } from '@/payload-types'
import { Input } from '@/components/ui/input'
import { Label } from '@/components/ui/label'
import { Textarea } from '@/components/ui/textarea'
import { toast } from 'sonner'

type Props = {
  className?: string
} & ContactFormProps

type FormState = 'idle' | 'loading' | 'success'

export const ContactForm = (props: Props) => {
  const [formState, setFormState] = useState<FormState>('idle')
  const [open, setOpen] = useState(false)

  const { form } = props
  const { title, fields, submitButtonLabel, id } = form as ContactFormData

  const handleSubmit = async (e: FormEvent<HTMLFormElement>) => {
    e.preventDefault()
    if (formState !== 'idle') return
    setFormState('loading')

    const formData = new FormData(e.currentTarget)

    console.log('...', [...formData.entries()])
    console.log('not spread', Object.entries(formData))

    const dataToSend = [...formData.entries()].map(([name, value]) => {
      return {
        field: name,
        value,
      }
    })
    console.log('datatosend', dataToSend)

    try {
      const submitResponse = await fetch('/api/form-submissions', {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({
          form: id,
          submissionData: dataToSend,
        }),
      })
      console.log(submitResponse)
      if (submitResponse.status !== 201) {
        setFormState('idle')
        toast('There was an issue with your submission. Please try again.')
      } else setFormState('success')
    } catch (err) {
      console.log(err)
      setFormState('idle')
      toast('There was an issue with your submission. Please try again.')
    }
  }

  const renderFields = (fields: Record<string, any>[]) => {
    return fields?.map((f) => {
      switch (f.blockType) {
        case 'text':
          return (
            <div key={f.name} className={`grid w-[${f.width}] max-w-sm items-center gap-1.5`}>
              <Label className="text-muted-foreground" htmlFor={f.name}>
                {f.label}
              </Label>
              <Input type="text" id={f.name} name={f.name} required={f.required} />
            </div>
          )
        case 'textarea':
          return (
            <div key={f.name} className={`grid w-[${f.width}] max-w-sm items-center gap-1.5`}>
              <Label className="text-muted-foreground" htmlFor={f.name}>
                {f.label}
              </Label>
              <Textarea
                placeholder="Type your message here."
                id={f.name}
                name={f.name}
                required={f.required}
                rows={4}
              />
            </div>
          )
        case 'email':
          return (
            <div key={f.name} className={`grid w-[${f.width}] max-w-sm items-center gap-1.5`}>
              <Label className="text-muted-foreground" htmlFor={f.name}>
                {f.label}
              </Label>
              <Input type="email" id={f.name} name={f.name} required={f.required} />
            </div>
          )
      }
    })
  }

  return (
    <div className="flex w-full items-center justify-center">
      <PopoverForm
        title={title}
        open={open}
        setOpen={setOpen}
        width="364px"
        height="auto"
        showCloseButton={formState !== 'success'}
        showSuccess={formState === 'success'}
        openChild={
          <form onSubmit={handleSubmit}>
            <div className="relative p-6 flex flex-col gap-3">{renderFields(fields || [])}</div>
            <div className="relative flex h-12 items-center px-[10px]">
              <PopoverFormSeparator />
              <div className="absolute left-0 top-0 -translate-x-[1.5px] -translate-y-1/2">
                <PopoverFormCutOutLeftIcon />
              </div>
              <div className="absolute right-0 top-0 translate-x-[1.5px] -translate-y-1/2 rotate-180">
                <PopoverFormCutOutRightIcon />
              </div>
              <PopoverFormButton loading={formState === 'loading'} text={submitButtonLabel || ''} />
            </div>
          </form>
        }
        successChild={
          <PopoverFormSuccess
            title="Feedback Received"
            description="Thank you for supporting our project!"
          />
        }
      />
    </div>
  )
}
